# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 
""" 
$Id: DiscussionItem.py,v 1.14.4.2 2004/09/01 05:06:38 fafhrd91 Exp $
"""

import traceback
from ExtensionClass import Base
from DateTime import DateTime
from Globals import Persistent, InitializeClass
import AccessControl
from AccessControl import ClassSecurityInfo
from Acquisition import Implicit, aq_base, aq_inner, aq_parent

from OFS.SimpleItem import SimpleItem
from OFS.Traversable import Traversable
from Products.CMFCore.CMFCorePermissions import \
     View, AccessContentsInformation, ReplyToItem, ManagePortal
from Products.CMFCore.utils import getToolByName, _checkPermission, _getViewFor
from Products.CMFCore.interfaces.Discussions import Discussable
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.CMFCatalogAware import CMFCatalogAware

from Products.Archetypes.public import *
from Products.Archetypes.Referenceable import Referenceable
from Products.Archetypes.CatalogMultiplex import CatalogMultiplex
from Products.Archetypes.interfaces.referenceable import IReferenceable

from utils import cb_isinstance
from ForumPermissions import AddTopicReply
from ForumContentTopic import ForumContentTopic
from ForumMessage import ForumMessage

class ForumDiscussionError(Exception):
    pass


class DiscussionItemWrapper( PortalContent ):
    meta_type = 'DiscussionItemWrapper'
    
    security = ClassSecurityInfo()

    def __init__(self, message, talkback):
        self.id = message.getId()
        self.message = message
        self.portal_type = 'Discussion Item'
        self.talkback = talkback

    def getId(self):
        """ """
        return self.id

    security.declareProtected(View, 'title_or_id')
    def title_or_id(self):
        """ """
        return self.message.Title()

    security.declareProtected(View, 'Creator')
    def Creator(self):
        """ """
        return self.message.Creator()

    security.declareProtected(View, 'Title')
    def Title(self):
        """ """
        return self.message.Title()

    security.declareProtected(View, 'CookedBody')
    def CookedBody(self):
        """ """
        request = self.REQUEST
        message = self.message.getMessage()
        topic = message.getTopic()
        topic.set_read_status(message)

        portal = getToolByName(self, 'portal_url').getPortalObject()
        return portal.forum_message_template(message=message)
    
    security.declareProtected(View, 'modified')
    def modified(self):
        """ """
        return self.message.modified()

    security.declareProtected(View, 'inReplyTo')
    def inReplyTo( self, REQUEST=None ):
        """
            Return the Discussable object to which we are a reply.

            Two cases obtain:

              - We are a "top-level" reply to a non-DiscussionItem piece
                of content;  in this case, our 'in_reply_to' field will
                be None.
            
              - We are a nested reply;  in this case, our 'in_reply_to'
                field will be the ID of the parent DiscussionItem.
        """
        obs = self.message.getRefs('parent')
        if obs and (obs[0].meta_type != ForumContentTopic.meta_type):
            return self.talkback.getReply(obs[0].getId())
        else:
            return None

    security.declareProtected(View, 'parentsInThread')
    def parentsInThread( self, size=0 ):
        """
            Return the list of items which are "above" this item in
            the discussion thread.

            If 'size' is not zero, only the closest 'size' parents
            will be returned.
        """
        parents = []
        current = self.message
        while not size or len( parents ) < size:
            p = current.getRefs('parent')
            if p and (p[0].meta_type != ForumContentTopic.meta_type):
                parent = p[0]
            else:
                break
            
            parents.insert( 0, self.talkback.getReply(parent.getId()) )
            current = parent
        return parents

    def __call__(self):
        '''
        Invokes the default view.
        '''
        view = _getViewFor(self)
        if getattr(aq_base(view), 'isDocTemp', 0):
            return apply(view, (self, self.REQUEST))
        else:
            return view()

    index_html = None  # This special value informs ZPublisher to use __call__

    security.declareProtected(View, 'view')
    def view(self):
        '''
        Returns the default view even if index_html is overridden.
        '''
        return self()

InitializeClass(DiscussionItemWrapper)


class ForumDiscussionContainer( Referenceable, CatalogMultiplex, SimpleItem ):
    """ """
    meta_type = 'ForumDiscussionContainer'

    __implements__ = (Discussable, IReferenceable)

    security = ClassSecurityInfo()

    _v_temps = None

    Subject = []

    def __init__(self, old_talkback):
        self.id = 'talkback'
        self.old_talkback = old_talkback

    def manage_afterAdd(self, item, container):
        Referenceable.manage_afterAdd(self, item, container)
        CatalogMultiplex.manage_afterAdd(self, item, container)

        if hasattr(aq_base(self), 'old_talkback'):
            old_talkback = self.old_talkback
            if not old_talkback:
                pass
            elif type(old_talkback) == type(''):
                # convert old style CMFBoard discussion
                try:
                    topic_obs = [o for o in
                                 self.archetype_tool.lookupObject(self.old_talkback)
                                 if o is not None]
                    if topic_obs:
                        topic_obs[0].addReference(self, 'cmfboard_topic')
                except:
                    pass
            elif old_talkback.meta_type == '':
                # convert default discussion tool
                pass

            del self.old_talkback

    def manage_afterClone(self, item):
        Referenceable.manage_afterClone(self, item)
        CatalogMultiplex.manage_afterClone(self, item)

    def manage_beforeDelete(self, item, container):
        try:
            topic = self._get_topic()
            if topic is not None:
                topic.manage_deleteMessage()
        except:
            pass

        Referenceable.manage_beforeDelete(self, item, container)
        CatalogMultiplex.manage_beforeDelete(self, item, container)

    security.declareProtected(View, 'getId')
    def getId(self):
        """ """
        return 'talkback'

    def __bobo_traverse__(self, REQUEST, name):
        # fix some bug in this code
        talkback = aq_inner(self)

        try:
            return talkback.getReply(name) 
        except:
            pass

        return getattr(self, name, )

    security.declareProtected(ManagePortal, 'deleteReply')
    def deleteReply( self, reply_id ):
        """ Remove a reply from this container """
        topic = self._get_topic()
        topic.manage_deleteReply([reply_id])

    security.declareProtected(View, 'hasReplies')
    def hasReplies( self, content_obj ):
        """ """
        try:
            return self._get_topic().replyCount()
        except:
            return 0

    security.declareProtected(View, 'replyCount')
    def replyCount( self, content_obj ):
        """ """
        try:
            return self._get_topic().replyCount()
        except:
            return 0

    security.declareProtected(View, 'getReply')
    def getReply(self, id):
        """ """
        topic = self._get_topic()

        #_v_temps = self._v_temps
        #if _v_temps is None:
        #    _v_temps = {}

        #if _v_temps.has_key(id):
        #    reply = _v_temps[id]
        #    reply.talkback = self
        #    return reply.__of__(self)
        #else:
        r = DiscussionItemWrapper(topic._getOb(id), self)
        #    _v_temps[id] = r
        #self._v_temps = _v_temps

        return r.__of__(self)

    security.declareProtected(ReplyToItem, 'createReply')
    def createReply(self, title, text, Creator=None):
        """ """
        topic = self._get_topic()

        parent = aq_parent(self)

        if cb_isinstance(parent, DiscussionItemWrapper):
            reply_to = parent.message.UID()
        else:
            reply_to = topic.UID()

        msg = topic.manage_addReply(title, text, Creator=Creator, reply_to=reply_to)

        id = msg.getId()
        reply = DiscussionItemWrapper(msg, self)

        _v_temps = self._v_temps
        if _v_temps is None:
            _v_temps = {}
        _v_temps[id] = reply
        self._v_temps = _v_temps

        return id

    security.declareProtected(View, 'getReplies')
    def getReplies(self):
        """
        Return a sequence of the DiscussionResponse objects which are
        associated with this Discussable

        Permissions: View
        Returns: sequence of DiscussionResponses
        """
        parent = aq_parent(self)

        talkback = aq_inner(self)

        if cb_isinstance(parent, DiscussionItemWrapper):
            replies = parent.message.getBRefs('parent')
        else:
            topic = self._get_topic()
            if topic is None:
                return []
            replies = topic.getBRefs('parent')

        _v_temps = self._v_temps
        if _v_temps is None:
            _v_temps = {}
            
        result = []
        for msg in replies:
            r = DiscussionItemWrapper(msg, self)
            _v_temps[msg.getId()] = r
            result.append(r.__of__(talkback))

        self._v_temps = _v_temps

        return result

    security.declareProtected(View, 'quotedContents')
    def quotedContents(self):
        """
        Return this object's contents in a form suitable for inclusion
        as a quote in a response.  The default implementation returns
        an empty string.  It might be overridden to return a '>' quoted
        version of the item.
        """

    security.declarePrivate( '_getReplyResults' )
    def _getReplyResults(self):
        """
        Return the ZCatalog results that represent this object's replies.

        Often, the actual objects are not needed.  This is less expensive
        than fetching the objects.

        Permissions: View
        Returns: sequence of ZCatalog results representing DiscussionResponses
        """


    security.declareProtected(View, 'isConfigured')
    def isConfigured(self, tp):
        """ """
        dtool = getToolByName(self, 'portal_discussion')

        if dtool.getForumForType(tp):
            return 1
        else:
            return 0

    security.declareProtected(View, 'isReplyAllowed')
    def isReplyAllowed(self, member):
        """ """
        forum = self._get_forum()
        if forum is not None:
            return _checkPermission(AddTopicReply, forum)
        else:
            return 0

    def _get_forum(self):
        """ """
        try:
            dtool = getToolByName(self, 'portal_discussion')
            atool = getToolByName(self, 'archetype_tool')

            tp = aq_parent(aq_inner(self)).getTypeInfo()

            forum_uid = dtool.getForumForType(tp.getId()) or dtool.getDefaultForum()
            return atool.lookupObject(forum_uid)
        except:
            traceback.print_exc()
            return None

    def _get_topic(self):
        """ """
        try:
            try:
                obs = self.getBRefs('cmfboard_topic')
                if obs:
                    return obs[0]
            except:
                pass

            dtool = getToolByName(self, 'portal_discussion')

            object = aq_parent(aq_inner(self))
            portal = aq_parent(aq_inner(dtool))

            # try discussion_filters
            current = object

            while 1:
                current = aq_parent(aq_inner(current))
            
                filter = getattr(current, 'discussion_filters', None)
                if filter is not None:
                    topic_rec = filter.createDiscussionFor(object, portal)
                    if topic_rec['status'] == None:
                        # goto next filter
                        continue
                    else:
                        # return created topic
                        return topic_rec['topic']
                else:
                    break

            # create default
            atool = getToolByName(self, 'archetype_tool')

            tp = object.getTypeInfo()

            forum_uid = dtool.getForumForType(tp.getId()) or dtool.getDefaultForum()
            forum = atool.lookupObject(forum_uid)
            if forum is None:
                raise ForumDiscussionError, 'CMFBord Discussion tool not configured'

            # create topic
            topic_ob = forum._addTopic(
                '', object.Title() or 'discussion', '', language=object.Language(),
                portal_type=ForumContentTopic.meta_type,
                content_uid = self.UID(),
                text_format=getattr(object,'text_format','text/plain'))
            return topic_ob
        except:
            traceback.print_exc()
            return None

    def _get_talkback(self):
        parent = self
        while aq_inner(parent).meta_type in ('ForumDiscussionContainer',
                                             'DiscussionItemWrapper'):
            parent = aq_parent(parent)

        return parent.talkback

    security.declareProtected(View, 'getContent')
    def getContent(self):
        """ """
        return aq_parent(aq_inner(self))


InitializeClass( ForumDiscussionContainer )
