# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz>
"""
$Id: Forum.py,v 1.78.4.3 2004/05/12 03:45:01 fafhrd91 Exp $
"""

import random, traceback, time
from Acquisition import aq_base, aq_inner
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
from Products.Archetypes.config import TOOL_NAME
from Products.Archetypes.Referenceable import Referenceable
from Products.CMFCore.utils import getToolByName, _checkPermission
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from BTrees.Length import Length

try:
    from Products.Archetypes.config import UUID_ATTR
except:
    UUID_ATTR = '_uid'

from config import *
from uihelpers import UIHelpers
from ForumCatalog import *
from ForumPermissions import *
from UserPreferences import UserPreferences
from PermissionsManagement import PermissionsManagement
from CacheHooks import CacheHooks
from ForumSchema import forumSchema
from ForumExtensibleSchema import ForumExtensibleSchema

try:
    from ForumMailReceiver import MailReceiver
except:
    traceback.print_exc()
    class MailReceiver:
        __implements__ = ()

try:
    from Subscribeable import Subscribeable
except:
    traceback.print_exc()
    from SubscribeableStub import Subscribeable


class ForumNB(ForumExtensibleSchema, BaseBTreeFolder, Subscribeable, CacheHooks,
              UserPreferences, PermissionsManagement, MailReceiver, UIHelpers):
    """ Forum """
    meta_type = 'ForumNB'
    archetype_name = 'ForumNB'

    __implements__ = (BaseBTreeFolder.__implements__ +
                      Subscribeable.__implements__ +
                      MailReceiver.__implements__)

    global_allow = 0
    filter_content_types = 1
    allowed_content_types = []
    content_icon = 'fcForum_icon.gif'

    schema = forumSchema

    actions = (
        { 'id': 'view',
          'name': 'View',
          'action': 'string:$object_url/forumnb_view',
          'permissions': (View,)
          },
        { 'id': 'edit',
          'name': 'Edit',
          'action': 'string:$object_url/base_edit',
          'permissions': (ManageForum,)
          },
        { 'id': 'subscribers',
          'name': 'Subscribers',
          'action': 'string:$object_url/forum_subscribers_form',
          'condition':
          "python:hasattr(portal, 'portal_subscriptions')",
          'permissions': (ManageForum,),
          },
        { 'id': 'forum_permissions',
          'name': 'Permissions',
          'action': 'string:$object_url/forum_user_permissions',
          'permissions': (ManageForum,),
          },
        )
    
    security = ClassSecurityInfo()

    __ac_roles__ = forumRoles

    security.declareProtected(ManageForum, 'processForm')

    def __init__(self, oid, **kw):
        BaseBTreeFolder.__init__(self, oid, **kw)

        self.hits = Length()
        self.replies = Length()

    security.declarePublic('isModerated')
    def isModerated(self):
        """ """
        return self.getField('moderated').get(self)

    security.declarePublic('isEditable')
    def isEditable(self):
        """ """
        return self.getField('editable').get(self)

    security.declareProtected(View, 'getWysiwygEditor')
    def getWysiwygEditor(self, member):
	""" """
        ed = self.getField('wysiwyg_editor').get(self)
        if not ed:
            return member.getProperty('wysiwyg_editor', 'None')
        else:
            return ed

    def _isPortalMember(self):
        """ """
        mtool = getToolByName(self, 'portal_membership')
        
        if mtool.isAnonymousUser():
            return 0
            
        member = mtool.getAuthenticatedMember()
        
        # Check if user is a portal member
        if mtool.getMemberById(member.getId()) is None:
            return 0
           
        return 1

    security.declareProtected(View, 'allowSubscription')
    def allowSubscription(self):
        """ """
        if not self._isPortalMember():
            return 0
           
        try:
            return Subscribeable.allowSubscription(self)
        except:
            return self.getField('allow_subscription').get(self)

    security.declareProtected(View, 'allowTopicSubscription')
    def allowTopicSubscription(self):
        """ """
        if not self._isPortalMember():
            return 0
           
        try:
            return Subscribeable.allowTopicSubscription(self)
        except:
            return self.getField('allow_topic_subscription').get(self)

    def all_meta_types(self, interfaces=None):
        """ """
        return []

    # rename copy support
    _copy = 0
    def _canCopy(self, op=0):
        if op == 1:
            return 1
        else:
            return 0

    def _notifyOfCopyTo(self, *args, **kwargs):
        self._copy = 1
        Referenceable._notifyOfCopyTo(self, *args, **kwargs)

    def manage_afterAdd(self, item, container):
        Subscribeable.manage_afterAdd(self, item, container)
        BaseBTreeFolder.manage_afterAdd(self, item, container)
        self._copy = 0

        mtool = getToolByName(self, 'portal_membership')
        user = mtool.getAuthenticatedMember()

        if not mtool.isAnonymousUser() and user:
            self.manage_setLocalRoles(user.getId(), [forumManager,])

    security.declarePrivate('initializeArchetype')
    def initializeArchetype(self, **kwargs):
        if kwargs.has_key('uid'):
            setattr(self, UUID_ATTR, kwargs['uid'])

        BaseBTreeFolder.initializeArchetype(self, **kwargs)

        if not hasattr(aq_base(self), ForumCatalog.id):
    	    addForumCatalog(self)

    security.declareProtected(View, 'getForum')
    def getForum(self):
        """ """
        return aq_inner(self)

    def _addTopic(self, id='', title='', text='', sticky_flag=0, topic_type=1,
                  portal_type='ForumTopic', **kwargs):
        # fixme need check portal_type

        kwargs.update({'title': title, 'text': text, 'topic_type': topic_type})
        if _checkPermission(ManageTopic, self):
            kwargs.update({'sticky_flag': sticky_flag})

        if not title:
            raise Exception, 'Null title'

	if not id:
    	    id = str(random.random())[2:]
        kwargs['id'] = id

        tp = self.portal_types.getTypeInfo(portal_type)
        if tp is None:
            raise 'ValueError', 'No such content type: ForumTopic'

        m = getattr(self.manage_addProduct['CMFBoard'], tp.factory)

        id = apply(m, (), kwargs)
        msg = tp._finishConstruction(self._getOb(id))
        return msg

    security.declareProtected(AddTopic, 'manage_addTopic')
    def manage_addTopic(self, title, text, sticky_flag=0, **kwargs):
        """ """
	if kwargs.has_key('id'):
	    del kwargs['id']
        msg = self._addTopic('', title, text, sticky_flag,
                             portal_type='ForumTopic', **kwargs)
        return msg

    security.declareProtected(AddPoll, 'manage_addPoll')
    def manage_addPoll(self, title, text, sticky_flag=0, **kwargs):
        """ """
	if kwargs.has_key('id'):
	    del kwargs['id']
        msg = self._addTopic('', title, text, sticky_flag,
                             portal_type='ForumPoll', **kwargs)
        return msg

    security.declareProtected(AddAnnouncement, 'manage_addAnnouncement')
    def manage_addAnnouncement(self, title, text, **kwargs):
        """ """
        kwargs['sticky_flag'] = 1
        return self._addTopic('', title, text, topic_type=4, **kwargs)

    security.declareProtected(View, 'manage_addSpecialTopic')
    def manage_addSpecialTopic(self, title, text, tp='admin', **kwargs):
        """ """
        kwargs['sticky_flag'] = 1
        if tp == 'admin':
            return self._addTopic('', title, text, topic_type=2, **kwargs)
        else:
            return self

    security.declareProtected(ManageForum, 'notifyStateChanged')
    def notifyStateChanged(self):
        """ """
        catalog = self.forum_catalog
        for brain in catalog():
            ob = brain.getObject()
            if ob is None:
                continue
            s = getattr(ob, '_p_changed', 0)
            catalog.reindexObject(ob, idxs=['allowedRolesAndUsers'])
            if s is None: ob._p_deactivate()

        self.reindexObjectSecurity()

    security.declareProtected(View, 'UID')
    def UID(self):
        """ """
        return BaseBTreeFolder.UID(self)

    security.declareProtected(View, 'Hits')
    def Hits(self):
        """ return hits and inc """
	try:
    	    self.hits.change(1)
    	    return self.hits()
	except:
	    return 0

    security.declareProtected(View, 'RawHits')
    def RawHits(self):
        """ """
	try:
    	    return self.hits()
	except:
	    return 0

    security.declareProtected(View, 'repliesCount')
    def repliesCount(self):
        """ return hits and inc """
        try:
            return aq_base(self).replies()
        except:
            self.replies = Length()
            return self.replies()

    security.declareProtected(ManageForum, 'changeRepliesCount')
    def changeRepliesCount(self, delta):
        """ return hits and inc """
        try:
            replies = aq_base(self).replies
        except:
            replies = Length()
            self.replies = replies
            
        replies.change(delta)
        if replies.value < 0:
           replies.set(0)

    #############################################
    ## CMFBoardCacheManager Hooks
    #############################################
    _cache_invalidates = {
        'message_new': ['forumnb_view'],
        'message_deleted': ['forumnb_view'],
        'message_changed': ['forumnb_view'],
        'message_updated': ['forumnb_view'],
        }

    
registerType(ForumNB)


def modify_fti(fti):
    for a in fti['actions']:
        if a['id'] in ('references', 'metadata'):
            a['visible'] = 0
    return fti
