# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 
"""
$Id: ForumMessage.py,v 1.91.4.5 2004/10/22 05:05:26 fafhrd91 Exp $
"""

import traceback, copy
from Acquisition import aq_inner, aq_parent, aq_base, aq_acquire
from AccessControl import ClassSecurityInfo, getSecurityManager
from OFS.Image import File
from Products.Archetypes.public import *
from Products.Archetypes.Field import decode, encode
from Products.Archetypes.interfaces.referenceable import IReferenceable
from Products.CMFCore.CMFCorePermissions import \
     View, ModifyPortalContent, ManagePortal, RequestReview
from Products.CMFCore.utils import format_stx, getToolByName
from DocumentTemplate.DT_Util import html_quote
from AccessControl.PermissionRole import rolesForPermissionOn

try:
    from Products.Archetypes.config import UUID_ATTR
except:
    UUID_ATTR = '_uid'

from utils import forum_parser, forum_html_tidy
from ForumPermissions import *
from ForumFileField import ForumFileField, ForumFileWidget
from ForumExtensibleSchema import MessageExtensibleSchema
from forum_codes import parser
from debug import log, log_exc
from config import CACHE_ID2

schema = BaseSchema + Schema((
    StringField('title', required=1, searchable=1,
                write_permission=ModifyReply,
                widget=StringWidget(label_msgid='label_title',
                                    description_msgid='help_title',
                                    i18n_domain='plone')),
    
    StringField('authorname', default='',
                write_permission=ModifyReply,
                widget=StringWidget(label='Author name',
                                    label_msgid='label_author_name',
                                    i18n_domain='CMFBoard')),
    
    ImageField('image',
               validators=('checkImageSize',),
               write_permission=ModifyPortalContent,
               allowable_content_types=('image/*',),
               sizes={'thumbnail': (128, 128),},
               widget=ImageWidget(label='Image')),
    
    ForumFileField('file',
                   validators=('checkFileSize',),
                   write_permission=ModifyPortalContent,
                   widget=ForumFileWidget(label='File')),
    
    StringField('message_icon', default='',
                write_permission=ModifyReply,
                widget=StringWidget(label='Message icon')),
    
    BooleanField('parsing_flag', default=1,
                 write_permission=ModifyReply,
                 widget=BooleanWidget()),

    TextField('text', default='', searchable=1,
              validators = ('checkTextSize',),
              accessor = 'EditableBody', mutator = 'setText',
              write_permission = ModifyReply,
              default_content_type = 'text/plain',
              default_output_type = 'text/html',
              allowable_content_types = ('text/structured',
                                         'text/restructured',
                                         'text/html',
                                         'text/plain'),
              widget=RichWidget(label='Body text')),
    
    IntegerField('signature', default=1,
                 write_permission=ModifyReply,
                 widget=IntegerWidget()),
    ))


class ForumMessage(MessageExtensibleSchema, BaseContent):
    """fcForum Object Class Python"""
    meta_type = 'ForumMessage'
    archetype_name = 'Forum Message'

    schema = schema

    global_allow = 0
    content_icon = 'discussionitem_icon.gif'

    actions = (
            { 'id': 'view',
              'name': 'View',
              'action': 'string:$object_url/forum_message',
              'permissions': (View,)
              },
            { 'id': 'edit',
              'name': 'Edit',
              'action': 'string:$object_url/forum_message_edit',
              'permissions': (ModifyReply,)
              },
            { 'id': 'reply',
              'name': 'Reply',
              'action': 'string:$object_url/forum_reply_form',
              'permissions': (AddTopicReply,)
             },
            { 'id': 'preview',
              'name': 'Preview',
              'action': 'string:$object_url/forum_message_preview',
              'permissions': (ModifyReply,)
              },
            { 'id': 'delete',
              'name': 'Delete',
              'action': 'string:$object_url/forum_message_delete_form',
              'permissions': (DeleteReply,)
              },
            )

    security = ClassSecurityInfo()
    security.setPermissionDefault(RequestReview, ['replyOwner'])
    security.declareProtected(ModifyReply, 'processForm')

    sended = 1
    message_icon = ''
    parsing_flag = 1
    signature = 1
    _anonymous = 0
    _initialized = 0

    security.declareProtected(RequestReview, '_test_test')
    def _test_test(self):
        pass

    def _setId(self, *args, **kwargs):
        pass

    def _canCopy(self, op=0):
        return 0

    def initializeArchetype(self, **kwargs):
        if kwargs.has_key('uid'):
            setattr(self, UUID_ATTR, kwargs['uid'])

        try:
            kwargs['file'].seek(0)
        except:
            pass

        try:
            if kwargs.has_key('parent'):
                parent = kwargs['parent']
                if IReferenceable.isImplementedBy(parent):
                    self.addReference(parent, 'parent')
        except:
            pass

        # process text
        set_text = 0
        try:
            text = kwargs['text']
            text_format = kwargs['text_text_format']

            del kwargs['text']
            del kwargs['text_text_format']
            set_text = 1
        except:
            pass
        
        BaseContent.initializeArchetype(self, **kwargs)

        if set_text:
            self.setText(text, mimetype=text_format)

	self.reindexObject(('SearchableText', 'Descrption'))

    security.declareProtected(ModifyPortalContent, 'allowDiscussion')
    def allowDiscussion(self, allowDiscussion=None, **kw):
        """ override Archetype's allowDiscussion method to handle traceback """
        if allowDiscussion is not None:
            try:
                allowDiscussion = int(allowDiscussion)
            except (TypeError, ValueError):
                allowDiscussion = allowDiscussion.lower().strip()
                d = {'on' : 1, 'off': 0, 'none':None, '':None}
                allowDiscussion = d.get(allowDiscussion, None)
        dtool = getToolByName(self, 'portal_discussion')
        try:
            dtool.overrideDiscussionFor(self, allowDiscussion)
        except:
            pass

    _saved_refs = 0
    _saved_refs_at13 = 0
    _set_owner = 1
    def manage_afterAdd(self, item, container):
        BaseContent.manage_afterAdd(self, item, container)

        # restore references
        try:
            refs = self._refs
        except:
            refs = []

        if self._saved_refs_at13:
            try:
                for rel, rs in refs.items():
                    for r in rs:
                        self.addReference(r, rel)
            except:
                traceback.print_exc()
            
            self._refs = ()
            self._saved_refs = 0
            self._saved_refs_at13 = 0
        else:
            atool = self.archetype_tool
            for r in refs:
                atool.addReference(self, r[0], r[1])
            self._refs = ()
            self._saved_refs = 0

        # update member posts count
        if not aq_base(self)._initialized:
            mtool = getToolByName(self, 'portal_membership')
            user = mtool.getAuthenticatedMember()

            if not mtool.isAnonymousUser() and user:
                try:
                    posts = int(user.getProperty('fc_posts', 0)) + 1
                except:
                    posts = 1
                try:
                    user.setMemberProperties({'fc_posts': posts})
                except:
                    pass

                if self._set_owner:
                    self.manage_permission(RequestReview, (replyOwner,), 0)
                    self.manage_setLocalRoles(user.getId(), [replyOwner,])

	    if mtool.isAnonymousUser():
		self._anonymous = 1

            # change forum replies counter
            if self.meta_type == 'ForumMessage':
                self.getForum().changeRepliesCount(1)

            self._initialized = 1

    def manage_beforeDelete(self, item, container):
        # need new code for this!!!
        if not self._saved_refs:
            try:
                self._refs = copy.copy(self.archetype_tool.refs.get(self.UID(), []))
                self._saved_refs = 1
            except:
                # AT-1.3
                refs = {}
                for rel in self.getRelationships():
                    refs[rel] = self.getRefs(rel)
                    
                self._refs = refs
                self._saved_refs = 1
                self._saved_refs_at13 = 1
                

        # check if remove or move
        if not (hasattr(item, '_copy') and item._copy):
            try:
                # remove children
                brefs = self.getBRefs('parent')
                if type(brefs) is not (type([]) or type(())):
                    brefs = [brefs,]
                children = [ob.getId() for ob in brefs]
                aq_parent(self).manage_deleteReply(children)
            except:
                pass

            mtool = self.portal_membership
            user = mtool.getMemberById(self.Creator())
            
            if user is not None:
                try:
                    posts = int(user.getProperty('fc_posts', 1)) - 1
                    if posts >= 0:
                        user.setMemberProperties({'fc_posts': posts})
                except:
                    user.setMemberProperties({'fc_posts': 0})

            # change forum replies counter
            if aq_base(self)._initialized:
                if self.meta_type == 'ForumMessage':
                    self.getForum().changeRepliesCount(-1)

            self._initialized = 0

        BaseContent.manage_beforeDelete(self, item, container)


    security.declareProtected(DeleteReply, 'manage_deleteMessage')
    def manage_deleteMessage(self, REQUEST=None):
        """ """
        aq_parent(self).manage_deleteReply([self.getId()])

        try:
            self.update_cache('message_deleted')
        except:
            pass

        return aq_parent(self)

    security.declareProtected(View, 'Title')
    def Title(self):
        """ """
        return self.getField('title').get(self)

    security.declareProtected(View, 'Description')
    def Description(self, mimetype='text/plain'):
        """ """
        return self.getField('description').get(self, mimetype=mimetype)

    security.declareProtected(View, 'CookedBody')
    def CookedBody(self, encoding=None):
        """ """
        if hasattr(aq_base(self), 'cooked_text'):
            return encode(self.cooked_text, self, encoding=encoding)
        else:
            return ''

    security.declareProtected(View, 'EditableBody')
    def EditableBody(self):
        """ """
        return self.getField('text').get(self)

    security.declareProtected(View, 'getParsingFlag')
    def getParsingFlag(self, REQUEST=None):
        """ """
        forum = self.getForum()
        forum_parsing_flag = forum.getField('parsing_flag').get(forum)

        if not forum_parsing_flag:
            if REQUEST is not None:
                return REQUEST.get(
                    'parsing_flag',
                    self.getField('parsing_flag').get(self))
            else:
                return self.getField('parsing_flag').get(self)
        elif forum_parsing_flag == 'on':
            return 1
        else:
            return 0

    security.declareProtected(View, 'getText')
    def getText(self, **kwargs):
        """ """
        return self.getField('text').get(self, **kwargs)

    security.declareProtected(ModifyReply, 'setText')
    def setText(self, value, mimetype='', **kwargs):
        """ """
        text_field = self.getField('text')

        if not mimetype:
            mimetype = text_field.getContentType(self)

        if mimetype == 'text/html':
            # check html for xhtml
            value = decode(value, self, **kwargs)
            errors, value, errordata = forum_html_tidy(self, value)

        text_field.set(self, value, mimetype=mimetype)

        # set cooked text and parse smiles, etc
        cooked_text = decode(
            text_field.get(self, mimetype='text/html'), self, **kwargs)

        if self.getParsingFlag(self.REQUEST):
            if mimetype == 'text/plain':
                self.cooked_text = forum_parser(self, value, 1, mimetype, 1)
            else:
                self.cooked_text = forum_parser(self, cooked_text, 1, mimetype)
        else:
            self.cooked_text = cooked_text
        
        # set description
        desc = decode(text_field.get(self, mimetype='text/plain'), self)
        desc = parser.feed(desc, self, 'desc', size_limit=150)

        self.getField('description').set(self, desc, mimetype='text/plain')

        try:
            self.update_cache('message_updated')
        except:
            pass

    security.declareProtected(View, 'Creator')
    def Creator(self):
        """ """
	if hasattr(aq_base(self), 'creator'):
	    if self.creator == 'Anonymous User':
		self._anonymous = 1
		del self.creator
	if self._anonymous:
            authorname = self.getField('authorname').get(self).strip()
            if authorname != '':
                return authorname
            else:
                return 'Anonymous User'
	else:
	    return BaseContent.Creator(self)

    security.declareProtected(View, 'isAnonymousCreator')
    def isAnonymousCreator(self):
        """ """
        return self._anonymous

    security.declareProtected(View, 'getParent')
    def getParent(self):
        """ """
        obs = self.getRefs('parent')
        try:
            return obs[0]
        except:
            return None

    security.declareProtected(View, 'getChildren')
    def getChildren(self, sort_on='created', reverse=0):
        """ """
        seq = []
        brefs = self.getBRefs('parent')
        if type(brefs) is not (type([]) or type(())):
            brefs = [brefs,]
        for ob in brefs:
            if ob is not None:
                attr = getattr(ob, sort_on)
                seq.append((attr(), ob))

        seq.sort()
        if reverse:
            seq.reverse()
        return [r[1] for r in seq]

    security.declareProtected(View, 'hasImage')
    def hasImage(self):
        """ """
        if hasattr(aq_base(self), 'image') and (self.image.getSize() > 0):
            return 1
        else:
            return 0

    security.declareProtected(View, 'getImage')
    def getImage(self):
        """ """
        if hasattr(aq_base(self), 'image'):
            return self.image
        else:
            return None

    security.declareProtected(View, 'getMessage')
    def getMessage(self):
        """ """
        return aq_inner(self)

    security.declareProtected(View, 'getSignature')
    def getSignature(self, portal=None):
        """ """
        if portal is None:
            portal = getToolByName(self, 'portal_url').getPortalObject()
        
        cache = None
        try:
            manager = aq_acquire(portal, CACHE_ID2, containment=1, default=None)
        except:
            manager = None
        if manager is not None:
            cache = manager.ZCacheManager_getCache()
            if cache is not None:
                uid = self.UID()
                data = cache.ZCache_get(portal, 'signature', keywords={'UID': uid})
                if data is not None:
                    return data

        sig = ''
        signature = aq_base(self).signature
        if signature and not self._anonymous:
            try:
                signature = int(signature) - 1
            except:
                self.signature = 0
                signature = 0
            mtool = portal.portal_membership

            member = mtool.getMemberById(self.Creator())
            if member:
                member_name = member.getUserName()

                sigs = member.getProperty('signatures', [])
                try:
                    sig = sigs[signature]
                except:
                    sig = ''

        if cache is not None:
            cache.ZCache_set(portal, sig, 'signature', keywords={'UID': uid})

        return sig

    security.declareProtected(View, 'notifyStateChanged')
    def notifyStateChanged(self, sci):
        """ """
        sended = self.sended

        if sci.old_state.getId() == 'initial':
            sended = 0
        
        new_state = sci.new_state.getId()

        try:
            # send notification
            if not sended:
                if new_state in ('published','visible','announce','admin','published'):
                    # send message
                    self.sended = 1
                    self.getForum().sendMessage(self, self.getParent())

                elif new_state == 'waitreview':
                    # send notification about moderation
                    self.sended = 0
                    self.getForum().sendModeratedNotification(self, self.getParent())
        except:
            #traceback.print_exc()
            log_exc('Sending email to subscribers faild.')

        #ctool = getToolByName(self, 'portal_catalog')
	#print ctool
        #ctool.indexObject(self)

registerType(ForumMessage)
