# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 
"""
$Id: ForumPoll.py,v 1.19.4.6 2004/06/07 05:41:51 fafhrd91 Exp $
"""

import types, operator

from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import View
from BTrees.Length import Length
from BTrees.IOBTree import IOBTree 
from BTrees.OOBTree import OOBTree

from config import *
from utils import cb_isinstance
from ForumTopic import ForumTopic
from ForumPermissions import AddTopicReply, ModifyTopic
from DateTime import DateTime


class ForumPoll(ForumTopic):
    """ forum poll topic based on MPoll product """
    meta_type = 'ForumPoll'
    archetype_name = 'Forum Poll'

    global_allow = 0
    filter_content_types = 1
    allowed_content_types = []
    content_icon = 'fcPoll_icon_noans.gif'

    schema = ForumTopic.schema + Schema((
        LinesField('answers', required=1, searchable=1,
                   write_permission=ModifyTopic,
                   widget=LinesWidget(description="A list of alternatives. One alternative per row.",
                                      label_msgid="label_answers",
                                      description_msgid="help_answers",
                                      i18n_domain="mpoll"),
                   ),
        ))
    
    actions = (
            {'id': 'view',
             'name': 'View',
             'action': 'string:$object_url/forum_topic',
             'permissions': (View,)
             },
            { 'id': 'edit',
              'name': 'Edit',
              'action': 'string:$object_url/forum_poll_edit',
              'permissions': (ModifyTopic,)
              },
            { 'id': 'reply',
              'name': 'Reply',
              'action': 'string:$object_url/forum_reply_form',
              'permissions': (AddTopicReply,)
              },
            { 'id': 'preview',
              'name': 'Preview',
              'action': 'string:$object_url/forum_message_preview',
              'permissions': (ModifyTopic,)
              },
            { 'id': 'delete',
              'name': 'Delete',
              'action': 'string:$object_url/forum_message_delete_form',
              'permissions': (ModifyTopic,)
              },         
            )


    security = ClassSecurityInfo()

    # poll
    topic_type = 3

    def __init__(self, oid, **kwargs):
        ForumTopic.__init__(self, oid, **kwargs)
        self._votes = IOBTree()
        self._member_votes = OOBTree()

    security.declareProtected(View, 'CookedBody')
    def CookedBody(self, encoding=None):
        """ """
        msg = ForumTopic.CookedBody(self, encoding)
        return self.forum_poll_view(message=msg)

    security.declareProtected(View, 'isOpen')
    def isOpen(self):
        """ """
        wtool = getToolByName(self, 'portal_workflow')
        return wtool.getInfoFor(self,'review_state','') != 'locked'
    
    security.declareProtected(View, 'getVote')
    def getVote(self):
        """get the vote"""
        mtool = getToolByName(self, 'portal_membership')
        
        if not hasattr(self, '_member_votes'):
            self._member_votes = OOBTree() # Add for compatibility with old version maybe detect in install in the future

        if mtool.isAnonymousUser():
	    try:
                request = self.REQUEST
        	return int(request.cookies.get(self.UID()))
	    except:
		return None
        else:
            member_id = mtool.getAuthenticatedMember().getId()
            return self._member_votes.get(member_id, None)
            
    security.declareProtected(View, 'unvote')
    def unvote(self):
        """Cancel a vote"""
        if not self.isOpen():
            return

        answer = self.getVote()
        
        if not answer:
            return

        if not cb_isinstance(answer, types.IntType):
            answer = int(answer)

        if not self._votes.has_key(answer):
            return

        self._votes[answer].change(-1)
        
        mtool = getToolByName(self, 'portal_membership')
        if mtool.isAnonymousUser():
            response = self.REQUEST.RESPONSE
            response.expireCookie(self.UID(), path='/')
        else:
            member_id = mtool.getAuthenticatedMember().getId()
            del self._member_votes[member_id]
        
    security.declareProtected(View, 'vote')
    def vote(self, answer):
        """Register a vote"""
        if not self.isOpen():
            return

        if not cb_isinstance(answer, types.IntType):
            answer = int(answer)
    
        if not self._votes.has_key(answer):
            self._votes[answer] = Length()

        self._votes[answer].change(1)

        mtool = getToolByName(self, 'portal_membership')
        if mtool.isAnonymousUser():
            response = self.REQUEST.RESPONSE

            e = (DateTime('GMT') + 365).rfc822()
            response.setCookie(self.UID(), str(answer), path='/', expires=e)
        else:
            member_id = mtool.getAuthenticatedMember().getId()
            self._member_votes[member_id] = answer
            
    security.declareProtected(View, 'getVotes')
    def getVotes(self):
        """Result of this poll"""

        result = {'total': reduce(
            operator.add, [votes() for votes in self._votes.values()] + [0,0])}

        scores = []

        i = 1
        for answer in self.getAnswers():
            votes = 0
            percent = 0

            if self._votes.has_key(i) and (result['total'] > 0):
                votes = self._votes[i]()
                percent = int(votes*100.0/result['total'])
            
            scores.append({'answer': answer,
                           'votes': votes,
                           'percent': percent})
            i = i+1
            
        result['scores'] = scores

        return result


registerType(ForumPoll)
