# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nikolay Kim <fafhrd@legco.biz>
"""
$Id: ForumTopic.py,v 1.69.2.7 2004/10/22 05:05:26 fafhrd91 Exp $
"""

import random, traceback
from DateTime import DateTime
from AccessControl import ClassSecurityInfo
from AccessControl.SecurityManagement import newSecurityManager
from Acquisition import aq_parent, aq_inner, aq_base
from Products.Archetypes.public import *
from Products.Archetypes.config import TOOL_NAME
from Products.CMFCore.CMFCorePermissions import \
     View, ModifyPortalContent, ManageProperties, ManagePortal, RequestReview
from Products.CMFCore.utils import getToolByName
from BTrees.Length import Length

from config import *
from ForumMessage import ForumMessage
from ForumPermissions import *
from ForumFileField import ForumFileField, ForumFileWidget
from ForumExtensibleSchema import MessageExtensibleSchema

try:
    from Subscribeable import Subscribeable
except:
    from SubscribeableStub import Subscribeable


schema = BaseSchema + Schema((
    StringField('title', required=1, searchable=1,
                write_permission=ModifyTopic,
                widget=StringWidget(label_msgid='label_title',
                                    description_msgid='help_title',
                                    i18n_domain='plone')),

    StringField('authorname', default='',
                write_permission=ModifyReply,
                widget=StringWidget(label='Author name',
                                    label_msgid='label_author_name',
                                    i18n_domain='CMFBoard')),

    ImageField('image', default='',
               validators=('checkImageSize',),
               write_permission=ModifyTopicAttachments,
               allowable_content_types=('image/*',),
               sizes={'thumbnail': (128, 128),},
               widget=ImageWidget(label='Image')),

    ForumFileField('file', default='',
                   validators=('checkFileSize',),
                   write_permission=ModifyTopicAttachments,
                   widget=ForumFileWidget(label='File')),

    StringField('message_icon',
                write_permission=ModifyTopic,
                widget=StringWidget(label='Message icon')),

    BooleanField('sticky_flag', default=0,
                 mutator='_change_sticky_flag',
                 write_permission=ManageTopic,
                 widget=BooleanWidget(label='Sticky')),

    BooleanField('parsing_flag', default=1,
                 write_permission=ModifyTopic,
                 widget=BooleanWidget()),

    TextField('text', default='', searchable=1,
              validators = ('checkTextSize',),
              accessor = 'EditableBody', mutator = 'setText',
              write_permission = ModifyTopic,
              default_content_type = 'text/plain',
              default_output_type = 'text/html',
              allowable_content_types = ('text/structured',
                                         'text/restructured',
                                         'text/html',
                                         'text/plain'),
              widget=RichWidget(label='Body text')),

    IntegerField('signature', default=0,
                 write_permission=ModifyReply,
                 widget=IntegerWidget()),
    ))


class ForumTopic(MessageExtensibleSchema, BaseBTreeFolder, ForumMessage, Subscribeable):
    """ """
    meta_type = 'ForumTopic'
    archetype_name = 'Forum Topic'

    __implements__ = (BaseBTreeFolder.__implements__ + Subscribeable.__implements__)

    schema = schema

    global_allow = 0
    filter_content_types = 1
    allowed_content_types = []

    actions = (
            { 'id': 'view',
              'name': 'View',
              'action': 'string:$object_url/forum_topic',
              'permissions': (View,)
              },
            { 'id': 'edit',
              'name': 'Edit',
              'action': 'string:$object_url/forum_message_edit',
              'permissions': (ModifyTopic,)
              },
            { 'id': 'reply',
              'name': 'Reply',
              'action': 'string:$object_url/forum_reply_form',
              'permissions': (AddTopicReply,)
              },
            { 'id': 'link',
              'name': 'Link',
              'action': 'string:$object_url/forum_link_form',
              'permissions': (ModifyTopic,)
              },
            { 'id': 'preview',
              'name': 'Preview',
              'action': 'string:$object_url/forum_message_preview',
              'permissions': (ModifyTopic,)
              },
            { 'id': 'delete',
              'name': 'Delete',
              'action': 'string:$object_url/forum_message_delete_form',
              'permissions': (ModifyTopic,)
              },
            )

    content_icon = 'fcTopic_icon_noans.gif'

    security = ClassSecurityInfo()
    security.setPermissionDefault(RequestReview, [replyOwner])

    _canCopy = ForumMessage._canCopy
    Creator = ForumMessage.Creator
    Title = ForumMessage.Title
    Description = ForumMessage.Description
    allowDiscussion = ForumMessage.allowDiscussion

    topic_type = 1
    signature = 0

    # for bug in Archetypes
    allow_discussion = 0

    security.declareProtected(ModifyTopic, 'setText')
    security.declareProtected(ModifyTopic, 'processForm')

    def __init__(self, oid, **kwargs):
        BaseBTreeFolder.__init__(self, oid, **kwargs)
        ForumMessage.__init__(self, oid, **kwargs)

        self.hits = Length()

    def _change_sticky_flag(self, value):
        try:
            self.sticky_flag = int(value)
        except:
            self.sticky_flag = 0
        self.modification_date = DateTime()
        self.reindexObject(['sticky_flag', 'modified'])

    def _change_locked_flag(self, value):
        self.locked_flag = value
        self.reindexObject(['locked_flag'])

    def initializeArchetype(self, **kwargs):
        if kwargs.has_key('topic_type'): # and kwargs['topic_type'] in topicTypeList:
            self.topic_type = kwargs['topic_type']

        BaseBTreeFolder.initializeArchetype(self, **kwargs)
        ForumMessage.initializeArchetype(self, **kwargs)

    def edit(self, **kwargs):
        """
        this is just an alias in Archetypes for the update method but we redefine it
        here the edit method in PortalFolder has being called which only accepts the
        title, and description arguements.
        """
        self.update(**kwargs)

    def all_meta_types(self, interfaces=None):
        """ """
        return []

    _set_owner = 0
    def manage_afterAdd(self, item, container):
        BaseBTreeFolder.manage_afterAdd(self, item, container)
        ForumMessage.manage_afterAdd(self, item, container)

        # update member posts count
        if not (hasattr(item, '_copy') and item._copy):
            mtool = getToolByName(self, 'portal_membership')
            user = mtool.getAuthenticatedMember()

            if not mtool.isAnonymousUser() and user:
                self.manage_permission(RequestReview, (topicOwner,), 0)
                self.manage_setLocalRoles(user.getId(), [topicOwner,])

                if (user.getProperty('fc_autosubscribe', 0) and
                    self.getForum().allowTopicSubscription()):
                    self.subscribe()

    def manage_beforeDelete(self, item, container):
        BaseBTreeFolder.manage_beforeDelete(self, item, container)
        ForumMessage.manage_beforeDelete(self, item, container)

    def _addReply(self, id, **kwargs):
        tool = getToolByName(self, 'portal_types')

        tp = tool.getTypeInfo('ForumMessage')
        if tp is None:
            raise 'ValueError', 'No such content type: ForumMessage'

        m = getattr(self.manage_addProduct['CMFBoard'], tp.factory)

        # need remove this code
        # it needed for first reply through email gate
        ################# start security hack #############################
        user_object = kwargs.get('user_object', None)
        if user_object is not None:
            newSecurityManager(None, user_object)
        ################### end security hack #############################
        
        id = apply(m, (id,), kwargs)
        msg = tp._finishConstruction(self._getOb(id))

        try:
            self.update_cache('message_new')
        except:
            pass

	return msg

    security.declareProtected(AddTopicReply, 'manage_addReply')
    def manage_addReply(self, title='', text='', reply_to='', **kwargs):
        """ """
        if not title:
            title = self.Title()
            if title[:3] != 'Re:':
                title = 'Re: ' + title

        kwargs['id'] = str(random.random())[2:]

        tool = getToolByName(self, TOOL_NAME)

        # get message to reply
        reply_to = tool.lookupObject(reply_to)
        if reply_to is not None:
            parent = reply_to
        else:
            # find last reply
            msgs = self.portal_catalog(
                portal_type='ForumMessage', path='/'.join(self.getPhysicalPath()),
                sort_on='created', sort_order='reverse')
            if len(msgs):
                parent = msgs[0].getObject()
            else:
                parent = self
                
        kwargs.update({'parent':parent, 'title':title, 'text': text})

	msg = self._addReply(**kwargs)

        self.modification_date = msg.created()
        self.reindexObject(['replyCount', 'modified'])

        return msg

    security.declareProtected('Delete objects', 'manage_delObjects')
    def manage_delObjects(self, ids=[], REQUEST=None):
        """ """
        return self.manage_deleteReply(ids, REQUEST)

    security.declareProtected(DeleteTopic, 'manage_deleteMessage')
    def manage_deleteMessage(self, REQUEST=None):
        """ """
        aq_parent(self).manage_delObjects([self.getId()])

        try:
            self.update_cache('message_deleted')
        except:
            pass
        
        return aq_parent(self)
        
    security.declarePrivate('manage_deleteReply')
    def manage_deleteReply(self, ids, REQUEST=None):
        """ """
        if type(ids) == type(''):
            ids = [ids]

        BaseBTreeFolder.manage_delObjects(self, ids)

    security.declareProtected(View, 'Hits')
    def Hits(self):
        """ return hits and inc """
        self.getForum().Hits()

	try:
    	    self.hits.change(1)
    	    return self.hits()
	except:
	    return 0

    security.declareProtected(View, 'RawHits')
    def RawHits(self):
        """ """
	try:
    	    return self.hits()
	except:
	    pass

    security.declareProtected(View, 'getTopic')
    def getTopic(self):
        """ """
        return aq_inner(self)

    security.declarePublic('getTopicType')
    def getTopicType(self):
        """ """
        return self.topic_type

    security.declareProtected(ModifyTopic, 'setSubject')
    def setSubject(self, subject):
        """ set subject and reindex """
        self.getField('subject').set(self, subject)
        self.reindexObject()

    security.declareProtected(View, 'replyCount')
    def replyCount(self, *args, **kwargs):
        """ for compatibility with default discussion tool """
        return self.objectCount()

    security.declareProtected(View, 'hasReplies')
    hasReplies = replyCount
    """ for compatibility with default discussion tool """


    ##########################################
    # forum_catalog support
    ##########################################
    security.declareProtected(ModifyPortalContent, 'indexObject')
    def indexObject(self):
        """ """
        BaseContent.indexObject(self)
        self.forum_catalog.indexObject(self)

    security.declareProtected(ModifyPortalContent, 'unindexObject')
    def unindexObject(self):
        """ """
        BaseContent.unindexObject(self)
        self.forum_catalog.unindexObject(self)

    security.declareProtected(ModifyPortalContent, 'reindexObject')
    def reindexObject(self, idxs=[]):
        """ """
        BaseContent.reindexObject(self, idxs)
        self.forum_catalog.reindexObject(self, idxs=idxs)


registerType(ForumTopic)
