# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.kz> 
"""
$Id: UserPreferences.py,v 1.16.4.1 2004/06/07 05:41:51 fafhrd91 Exp $
"""

import random, traceback, time
from cPickle import loads, dumps
from types import InstanceType

from ExtensionClass import Base
from Globals import InitializeClass
from Acquisition import aq_base, aq_acquire
from AccessControl import ClassSecurityInfo

from OFS.SimpleItem import SimpleItem
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.utils import getToolByName, _checkPermission

from ForumCatalog import *
from ForumPermissions import *
from config import CACHE_ID2
from debug import log_exc
from utils import cb_isinstance

class UserPreferences(SimpleItem):

    security = ClassSecurityInfo()

    security.declareProtected(View, 'get_user_role')
    def get_user_role(self, user, default=''):
        """ """
        user = getToolByName(self, 'portal_membership').getMemberById(user)
        try:
            if user:
                roles = user.getRoles()
                if ('Forum manager' in roles) or ('Manager' in roles):
                    return 'Manager'
                elif 'Forum reviewer' in roles:
                    return 'Reviewer'
                else:
                    return 'Member'
        except:
            pass

        return 'Anonymous'


    security.declareProtected(View, 'get_user_properties')
    def get_user_properties(self, user, portal=None, mtool=None):
        """ """
        user = str(user)

        if portal is None:
            portal = getToolByName(self, 'portal_url').getPortalObject()

        cache = None
        try:
            manager = aq_acquire(portal, CACHE_ID2, containment=1, default=None)
        except:
            manager = None
        if manager is not None:
            cache = manager.ZCacheManager_getCache()
            if cache is not None:
                data = cache.ZCache_get(portal, 'user_props', keywords={'user': user})
                if data is not None:
                    return data
            
        if mtool is None:
            mtool = portal.portal_membership
        
        data = {'fc_posts': 'n/a',
                'fc_emailtome': 0,
                'fc_forum_vm': 'flat',
                'fc_topic_vm': 'flat',
                'homepage': '',
                'fullname': user,
                'avatar': '',
                'anonymous': 1,
                }

        # maybe fix for LdapUserFolder
        if (user == 'None') or not user:
            return data
        
        user_ob = mtool.getMemberById(user)
        if user_ob:
            data['anonymous'] = 0
            if user_ob.getProperty('show_email', 0):
                email = user_ob.getProperty('email', '')
                new_email = ''.join(['&#%s;'%ord(ch) for ch in email])
                data['email'] = new_email
                data['email_tag'] = '<a href="mailto:%s">%s</a>'%(new_email, new_email)
            if user_ob.getProperty('show_im', 0):
                data['im_jabber'] = user_ob.getProperty('im_jabber', '')
                data['im_yahoo'] = user_ob.getProperty('im_yahoo', '')
                data['im_icq'] = user_ob.getProperty('im_icq', '')
                data['im_msn'] = user_ob.getProperty('im_msn', '')
                data['im_aim'] = user_ob.getProperty('im_aim', '')
                data['im_ichat'] = user_ob.getProperty('im_ichat', '')

            if (user_ob.getProperty('fc_emailtome', 0) and
                user_ob.getProperty('email', '')):
                data['fc_emailtome'] = 1

            data['fc_posts'] = user_ob.getProperty('fc_posts', 'n/a')
            data['fc_forum_vm'] = user_ob.getProperty('fc_forum_vm', 'flat')
            data['fc_topic_vm'] = user_ob.getProperty('fc_topic_vm', 'flat')

            data['first_login_time'] = user_ob.getProperty('first_login_time', '')
            data['last_login_time'] = user_ob.getProperty('last_login_time', '')
            data['location'] = user_ob.getProperty('location', '')
            data['occupation'] = user_ob.getProperty('occupation', '')
            data['interests'] = user_ob.getProperty('interests', '')
            data['homepage'] = user_ob.getProperty('homepage', '')
            data['timezone'] = user_ob.getProperty('timezone', '')

            if user_ob.getProperty('show_fullname', 0):
                data['fullname'] = user_ob.getProperty('fullname','') or user

            avatar = mtool.getPersonalPortrait(user_ob.getId())
            if avatar.getId() != 'defaultUser.gif':
                data['avatar'] = avatar.tag(width=75, height=100)

            # remove instances
            filtered = {}
            for k, v in data.items():
                if cb_isinstance(v, Base):
                    filtered[k] = ''
                else:
                    filtered[k] = v
                    
            data = filtered
            
            if cache is not None:
                try:
                    cache.ZCache_set(portal, data, 'user_props', keywords={'user': user})
                except:
                    log_exc('Unkown error')
                    
        return data


    security.declareProtected(View, 'get_read_status')
    def get_read_status(self, object, topic=0, REQUEST=None, mtool=None):
        """ """
        if REQUEST is None:
            REQUEST = self.REQUEST
        
        if hasattr(REQUEST, '_read_status'):
            read_status = REQUEST._read_status
        else:
            if mtool is None:
                mtool = getToolByName(self, 'portal_membership')
                
            if mtool.isAnonymousUser():
                return 1
            else:
                member = self.portal_membership.getAuthenticatedMember()
                try:
                    internal_data = loads(member.getProperty('fc_internal_data', ''))
                except:
                    internal_data = {}

                read_status = internal_data.get('read_status', {})
                read_status['last_login_time'] = member.getProperty('last_login_time')
                REQUEST._read_status = read_status

        data = read_status.setdefault('data', {})

        object_modified = object.modified()

        if read_status['last_login_time'] > object_modified:
            return 1

        if topic:
            topic_ob = object.getTopic()
            
            for ob in topic_ob.contentValues():
                if _checkPermission('View', ob):
                    status = self.get_read_status(ob, REQUEST=REQUEST)
                    if not status:
                        return 0

        if data.get(object.UID(), 0) > object_modified.timeTime():
            return 1

        return 0
    

    security.declareProtected(View, 'set_read_status')
    def set_read_status(self, object, REQUEST=None):
        """ """
        if REQUEST is None:
            REQUEST = self.REQUEST

        mtool = getToolByName(self, 'portal_membership')
        if mtool.isAnonymousUser():
            return

        member = self.portal_membership.getAuthenticatedMember()
        try:
            last_login = member.getProperty('last_login_time')
            last_login_time = last_login.timeTime()
        except:
            return

        try:
            internal_data = loads(member.getProperty('fc_internal_data', ''))
        except:
            internal_data = {}

        read_status = internal_data.get('read_status', {})
        read_status['last_login_time'] = last_login
        data = read_status.get('data', {})

        object_uid = object.UID()
        object_modified = object.modified()
        object_modified_time = object_modified.timeTime()

        # check is it really need save access time
        if ((data.get(object_uid, 0) > object_modified_time) or
            (last_login > object_modified)):
            return
        
        data[object.UID()] = time.time()

        # clear data
        new_data = {}
        try:
            for uid, object_time in data.items():
                if object_time > last_login_time:
                    new_data[uid] = object_time
        except:
            pass

        # save data
        read_status['data'] = new_data
        internal_data['read_status'] = read_status
        member.setMemberProperties({'fc_internal_data': dumps(internal_data,1)})

        REQUEST._read_status = read_status


InitializeClass(UserPreferences)
