#
# BoardTestCase
# based on PloneTestCase

# $Id: BoardTestCase.py,v 1.2 2003/11/11 19:14:01 fafhrd91 Exp $

from Testing import ZopeTestCase

ZopeTestCase.installProduct('CMFCore', quiet=1)
ZopeTestCase.installProduct('CMFDefault', quiet=1)
ZopeTestCase.installProduct('CMFCalendar', quiet=1)
ZopeTestCase.installProduct('CMFTopic', quiet=1)
ZopeTestCase.installProduct('DCWorkflow', quiet=1)
ZopeTestCase.installProduct('CMFQuickInstallerTool', quiet=1)
ZopeTestCase.installProduct('Archetypes', quiet=1)
ZopeTestCase.installProduct('MPoll', quiet=1)
ZopeTestCase.installProduct('CMFBoard')
ZopeTestCase.installProduct('CMFPlone', quiet=1)
ZopeTestCase.installProduct('MailHost', quiet=1)
ZopeTestCase.installProduct('PageTemplates', quiet=1)
ZopeTestCase.installProduct('PythonScripts', quiet=1)
ZopeTestCase.installProduct('ExternalMethod', quiet=1)

from AccessControl import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import noSecurityManager
from Acquisition import aq_base
import time

portal_name  = 'portal'
portal_owner = 'portal_owner'
portal_member = 'portal_member'

board_manager = 'fmanager'
board_reviewer = 'freviewer'
board_member = 'fmember'

default_user = ZopeTestCase.user_name


class BoardTestCase(ZopeTestCase.PortalTestCase):

    def getPortal(self):
        '''Returns the portal object.'''
        return self.app[portal_name]

    def loginPortalOwner(self):
        '''Use if you need to manipulate the portal itself.'''
        noSecurityManager()
        uf = self.app.acl_users
        user = uf.getUserById(portal_owner).__of__(uf)
        newSecurityManager(None, user)

    def loginPortalMember(self):
        '''Use if you need to manipulate the portal itself.'''
        noSecurityManager()
        uf = self.app.acl_users
        user = uf.getUserById(portal_member).__of__(uf)
        newSecurityManager(None, user)

def setupCMFBoard(app=None, id=portal_name, quiet=0):
    '''Creates a Plone site.'''
    if not hasattr(aq_base(app), id):
        _start = time.time()

        # Add user and log in
        uf = app.acl_users
        uf._doAddUser(portal_owner, '', ['Manager'], [])
        uf._doAddUser(portal_member, '', ['Member'], [])
        uf._doAddUser(board_manager, '', [], [])
        uf._doAddUser(board_member, '', ['Reviewer'], [])
        
        user = uf.getUserById(portal_owner).__of__(uf)
        newSecurityManager(None, user)
        # Add Plone Site
        factory = app.manage_addProduct['CMFPlone']
        factory.manage_addSite(id, '', create_userfolder=0)
        portal = app[id]

        # Add Quickinstaller
        factory = portal.manage_addProduct['CMFQuickInstallerTool']
        factory.addQuickInstallerTool()

        # Install Products
        portal.portal_quickinstaller.installProduct('MPoll')
        portal.portal_quickinstaller.installProduct('CMFBoard')

        # install 
	portal.invokeFactory('ForumFolder', 'forums')

        # Log out
        noSecurityManager()
        get_transaction().commit()
        if not quiet: ZopeTestCase._print('done (%.3fs)\n' % (time.time()-_start,))

def optimize():
    '''Significantly reduces portal creation time.'''
    # Don't compile expressions on creation
    def __init__(self, text):
        self.text = text
    from Products.CMFCore.Expression import Expression
    Expression.__init__ = __init__
    # Don't clone actions but convert to list only
    def _cloneActions(self):
        return list(self._actions)
    from Products.CMFCore.ActionProviderBase import ActionProviderBase
    ActionProviderBase._cloneActions = _cloneActions
    # Don't setup default directory views
    def setupDefaultSkins(self, p):
        from Products.CMFCore.utils import getToolByName
        ps = getToolByName(p, 'portal_skins')
        ps.manage_addFolder(id='custom')
        ps.addSkinSelection('Basic', 'custom')
    from Products.CMFDefault.Portal import PortalGenerator
    PortalGenerator.setupDefaultSkins = setupDefaultSkins
    # Don't setup Plone content (besides members folder)
    def setupPortalContent(self, p):
        from Products.CMFPlone.LargePloneFolder import addLargePloneFolder
        p.manage_delObjects('Members')
        addLargePloneFolder(p, 'Members')
        p.portal_catalog.unindexObject(p.Members)
        p.Members._setPortalType = 'Folder'
    from Products.CMFPlone.Portal import PloneGenerator
    PloneGenerator.setupPortalContent = setupPortalContent


#optimize()

# Create a Plone site in the test (demo-) storage
app = ZopeTestCase.app()
setupCMFBoard(app)
ZopeTestCase.close(app)

